<?php

namespace Drupal\block_style_plugins;

use Drupal\block_style_plugins\Plugin\BlockStyleInterface;
use Drupal\block_style_plugins\Plugin\BlockStyleManager;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for Form Alters where plugins will be used.
 */
abstract class BaseFormAlter implements ContainerInjectionInterface {

  use StringTranslationTrait;
  use IncludeExcludeStyleTrait;
  use FormEntityHelperTrait;
  use DependencySerializationTrait;

  /**
   * Instance of the Entity Repository service.
   *
   * @var \Drupal\block_style_plugins\Plugin\BlockStyleManager
   */
  protected $blockStyleManager;

  /**
   * The label for the default field grouping on the form.
   *
   * @var string
   */
  protected $groupName = 'Block Styles';

  /**
   * Construct method for SectionFormAlter.
   *
   * @param \Drupal\block_style_plugins\Plugin\BlockStyleManager $blockStyleManager
   *   A Block Style Manager instance.
   */
  public function __construct(BlockStyleManager $blockStyleManager) {
    $this->blockStyleManager = $blockStyleManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.block_style.processor')
    );
  }

  /**
   * Create a fieldset to store plugin fields within.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param string $plugin_id
   *   The ID of the plugin being instantiated.
   * @param array $plugin_definition
   *   A plugin definition.
   */
  public function createGroup(array &$form, string $plugin_id, array $plugin_definition): void {
    $group_machine_name = 'block_styles';

    // Create a custom group fieldset to contain style fields.
    if (!empty($plugin_definition['group'])) {
      $group_name = $plugin_definition['group'];
      $group_machine_name = preg_replace("/[^0-9a-z ]/m", "", strtolower($group_name));
      $group_machine_name = 'block_styles_' . preg_replace("/ /", "_", $group_machine_name);
      $form[$group_machine_name] = $this->createGroupFieldset($group_name);
    }

    // Create a fieldset to contain style fields.
    elseif (!isset($form['block_styles'])) {
      $group_name = $this->groupName;
      $form[$group_machine_name] = $this->createGroupFieldset($group_name);
    }

    $form['third_party_settings']['block_style_plugins'][$plugin_id] = [
      '#type' => 'container',
      '#group' => $group_machine_name,
    ];
  }

  /**
   * Create the Fieldset form array.
   *
   * @param string $group_name
   *   The label for the form fieldset.
   *
   * @return array
   *   The Render array for a form fieldset.
   */
  protected function createGroupFieldset(string $group_name): array {
    return [
      '#type' => 'fieldset',
      '#title' => $this->t($group_name),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#weight' => 0,
    ];
  }

  /**
   * Remove styles that have empty values.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param \Drupal\block_style_plugins\Plugin\BlockStyleInterface $style_plugin
   *   The style plugin.
   *
   * @return array
   *   All the styles that do not have empty values.
   */
  protected function removeEmptyStyles(array $form, BlockStyleInterface $style_plugin) {
    $plugin_id = $style_plugin->getPluginId();
    $styles = $style_plugin->getConfiguration();
    return array_filter($styles, function ($value, $key) use ($form, $plugin_id) {
      $field_type = $form['third_party_settings']['block_style_plugins'][$plugin_id][$key]['#type'] ?? NULL;
      if (!is_null($value) && $value !== '' && $field_type !== 'checkbox') {
        return TRUE;
      }
      elseif (!empty($value)) {
        return TRUE;
      }
      return FALSE;
    }, ARRAY_FILTER_USE_BOTH);
  }

  /**
   * Add Validate and Submit handlers to the form.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   */
  protected function addFormHandlers(array &$form): void {
    // Form Validate and Submit handlers can work different in blocks and
    // the layout builder embedded form so we want to apply our handler wherever
    // they are used.
    if (isset($form['actions']['submit']['#validate'])) {
      $form['actions']['submit']['#validate'][] = [$this, 'validateForm'];
    }
    else {
      $form['#validate'][] = [$this, 'validateForm'];
    }

    if (isset($form['actions']['submit']['#submit'])) {
      array_unshift($form['actions']['submit']['#submit'], [
        $this,
        'submitForm'
      ]);
    }
    else {
      $form['#submit'] = $form['#submit'] ?? [];
      array_unshift($form['#submit'], [$this, 'submitForm']);
    }
  }

}
