<?php

namespace Drupal\block_style_plugins;

use Drupal\Core\Form\SubformState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\layout_builder\SectionComponent;

/**
 * Base class for Block style plugins.
 */
class BlockFormAlter extends BaseFormAlter {

  /**
   * Alter a form and set block configuration.
   *
   * This code will be run as part of a form alter so that the current blocks
   * configuration will be available to the form.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param bool $layout_builder
   *   TRUE if the current form is the Layout Builder component settings.
   */
  public function alterForm(array &$form, FormStateInterface $form_state, $layout_builder = FALSE) {
    // Retrieve a list of style plugin definitions.
    $restrict_to = ($layout_builder) ? 'layout_builder' : 'block_layout';
    $style_plugins = $this->blockStyleManager->getBlockDefinitions($restrict_to);
    if (empty($style_plugins)) {
      return;
    }

    foreach ($style_plugins as $plugin_id => $plugin_definition) {
      if ($layout_builder && empty($plugin_definition['layout_builder_embed_form'])) {
        continue;
      }

      /** @var \Drupal\block_style_plugins\Plugin\BlockStyleInterface $style_plugin */
      $style_plugin = $this->blockStyleManager->createInstance($plugin_id);

      // Get the current block config entity.
      $entity = $this->getBlockConfigEntity($form_state);

      // Set properties and configuration.
      $block_plugin = $entity->getPlugin();
      $block_content_bundle_id = $this->getBlockContentBundle($form_state);

      // Find the plugin ID or block content bundle id.
      $block_plugin_id = $block_plugin->getPluginId();
      if ($block_content_bundle_id) {
        $block_plugin_id = $block_content_bundle_id;
      }

      // Check to see if this should only apply to includes or if it has been
      // excluded.
      if ($this->allowStyles($block_plugin_id, $plugin_definition)) {
        // Create containers to place each plugin style settings into the styles
        // fieldset.
        $this->createGroup($form, $plugin_id, $plugin_definition);

        $styles = $entity->getThirdPartySetting('block_style_plugins', $plugin_id, []);
        $style_plugin->setConfiguration($styles);

        // Allow plugins to add field elements to this form.
        if ($style_plugin instanceof PluginFormInterface) {
          $subform_state = SubformState::createForSubform($form['third_party_settings']['block_style_plugins'][$plugin_id], $form, $form_state);
          $form['third_party_settings']['block_style_plugins'][$plugin_id] += $style_plugin->buildConfigurationForm($form['third_party_settings']['block_style_plugins'][$plugin_id], $subform_state);
        }
      }
    }

    $this->addFormHandlers($form);
  }

  /**
   * Alter the component settings form in Layout Builder settings tray.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string $form_id
   *   Id of the current form.
   */
  public function layoutBuilderEmbedAlter(array &$form, FormStateInterface $form_state, $form_id) {
    if ($form_id == 'layout_builder_update_block' || $form_id == 'layout_builder_add_block') {
      $this->alterForm($form, $form_state, TRUE);
    }
  }

  /**
   * Validate the form.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function validateForm(array $form, FormStateInterface $form_state) {
    $style_settings = $form_state->getValue(['third_party_settings', 'block_style_plugins']);
    if (empty($style_settings)) {
      return;
    }

    // Retrieve a list of possible style plugin definitions.
    $available_plugins = $this->blockStyleManager->getBlockDefinitions();

    foreach ($style_settings as $plugin_id => $settings) {
      // Only instantiate plugins that are available.
      if (array_key_exists($plugin_id, $available_plugins)) {
        /** @var \Drupal\block_style_plugins\Plugin\BlockStyleInterface $style_plugin */
        $style_plugin = $this->blockStyleManager->createInstance($plugin_id);
        if ($style_plugin instanceof PluginFormInterface) {
          $subform_state = SubformState::createForSubform($form['third_party_settings']['block_style_plugins'][$plugin_id], $form, $form_state);
          $style_plugin->validateConfigurationForm($form['third_party_settings']['block_style_plugins'][$plugin_id], $subform_state);
        }
      }
    }
  }

  /**
   * Submit the form and save configuration.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array $form, FormStateInterface $form_state) {
    $style_settings = $form_state->getValue(['third_party_settings', 'block_style_plugins']);
    if (empty($style_settings)) {
      return;
    }

    // Retrieve a list of possible style plugin definitions.
    $available_plugins = $this->blockStyleManager->getBlockDefinitions();

    $component = $this->getBlockConfigEntity($form_state);

    foreach ($style_settings as $plugin_id => $styles) {
      // Only instantiate plugins that are available.
      if (array_key_exists($plugin_id, $available_plugins)) {
        /** @var \Drupal\block_style_plugins\Plugin\BlockStyleInterface $style_plugin */
        $style_plugin = $this->blockStyleManager->createInstance($plugin_id);
        $style_plugin->setConfiguration($styles);

        if ($style_plugin instanceof PluginFormInterface) {
          $subform_state = SubformState::createForSubform($form['third_party_settings']['block_style_plugins'][$plugin_id], $form, $form_state);
          $style_plugin->submitConfigurationForm($form['third_party_settings']['block_style_plugins'][$plugin_id], $subform_state);
          // Save any altered styles back to the plugin config.
          $altered_style = $form_state->getValue(['third_party_settings', 'block_style_plugins', $plugin_id]);
          $style_plugin->setConfiguration($altered_style);
        }

        // Embedded form on Layout Builder settings tray needs styles saved to
        // the component.
        if ($component instanceof SectionComponent) {
          // Remove any empty styles.
          $remaining_styles = $this->removeEmptyStyles($form, $style_plugin);
          if (empty($remaining_styles)) {
            $component->unsetThirdPartySetting('block_style_plugins', $plugin_id);
          }
          else {
            $component->setThirdPartySetting('block_style_plugins', $plugin_id, $style_plugin->getConfiguration());
          }
        }
      }
    }
  }

}
