<?php

namespace Drupal\block_style_plugins;

use Drupal\block_content\BlockContentInterface;
use Drupal\block_style_plugins\Form\ConfigureStyles;
use Drupal\Core\Form\FormStateInterface;
use Drupal\layout_builder\Form\AddBlockForm;
use Drupal\layout_builder\Form\UpdateBlockForm;

/**
 * Provides a helper for getting information from the entity being styled.
 */
trait FormEntityHelperTrait {

  /**
   * Instance of the Entity Repository service.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Instance of the Entity Type Manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Gets the Entity Repository service.
   *
   * @return \Drupal\Core\Entity\EntityRepositoryInterface
   *   The Entity Repository service.
   */
  protected function getEntityRepository() {
    if (!$this->entityRepository) {
      $this->entityRepository = \Drupal::service('entity.repository');
    }

    return $this->entityRepository;
  }

  /**
   * Gets the Entity Type Manager service.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The Entity Type Manager service.
   */
  protected function getEntityTypeManager() {
    if (!$this->entityTypeManager) {
      $this->entityTypeManager = \Drupal::service('entity_type.manager');
    }

    return $this->entityTypeManager;
  }

  /**
   * Get the block content bundle type.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return string
   *   Name of the Block Content bundle.
   */
  public function getBlockContentBundle(FormStateInterface $form_state) {
    $bundle = NULL;

    // Get the current block config entity.
    $block_content = $this->getBlockContent($form_state);

    if ($block_content) {
      $bundle = $block_content->bundle();
    }
    else {
      // If this is an inline block on first creation.
      $entity = $this->getBlockConfigEntity($form_state);
      /** @var \Drupal\Core\Block\BlockPluginInterface $block_plugin */
      $block_plugin = $entity->getPlugin();
      if ($block_plugin->getBaseId() == 'inline_block') {
        $bundle = $block_plugin->getDerivativeId();
      }
    }

    return $bundle;
  }

  /**
   * Get the Block Content entity.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return \Drupal\block_content\BlockContentInterface
   *   The Block Content entity.
   */
  public function getBlockContent(FormStateInterface $form_state) {
    // Get the current block config entity.
    $entity = $this->getBlockConfigEntity($form_state);

    /** @var \Drupal\Core\Block\BlockPluginInterface $block_plugin */
    $block_plugin = $entity->getPlugin();
    $base_id = $block_plugin->getBaseId();
    $block_content = NULL;

    if ($base_id == 'block_content') {
      $uuid = $block_plugin->getDerivativeId();
      $block_content = $this->getEntityRepository()->loadEntityByUuid('block_content', $uuid);
    }
    if ($base_id == 'inline_block') {
      $config = $block_plugin->getConfiguration();
      if (!empty($config['block_serialized'])) {
        $block_content = unserialize($config['block_serialized']);
      }
      elseif (!empty($config['block_revision_id'])) {
        $block_content = $this->getEntityTypeManager()
          ->getStorage('block_content')
          ->loadRevision($config['block_revision_id']);
      }
    }
    return $block_content;
  }

  /**
   * Get the config entity of the entity being styled from the form state.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return \Drupal\block\Entity\Block|\Drupal\layout_builder\SectionComponent|\Drupal\layout_builder\Section
   *   The entity for the current Block, SectionComponent, or Section.
   */
  public function getEntity(FormStateInterface $form_state) {
    // Try to get a block/component entity.
    $entity = $this->getBlockConfigEntity($form_state);

    // Try to get a section entity if this is not a block.
    if (!$entity) {
      $entity = $this->getLayoutSection($form_state);
    }

    return $entity;
  }

  /**
   * Get the Block config entity from the form state.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return \Drupal\block\Entity\Block|\Drupal\layout_builder\SectionComponent
   *   The entity for the current Block or layout builder SectionComponent.
   */
  public function getBlockConfigEntity(FormStateInterface $form_state) {
    /** @var Drupal\block\BlockForm $form_object */
    $form_object = $form_state->getFormObject();
    $entity = NULL;

    // Get the current block config entity.
    if ($form_object instanceof UpdateBlockForm) {
      /** @var \Drupal\layout_builder\SectionComponent $entity */
      $entity = $form_object->getCurrentComponent();
    }
    elseif ($form_object instanceof AddBlockForm) {
      /** @var \Drupal\layout_builder\SectionComponent $entity */
      $entity = $form_state->get('layout_builder__component');
    }
    elseif ($form_object instanceof ConfigureStyles) {
      /** @var \Drupal\layout_builder\SectionComponent $entity */
      $entity = $form_object->getComponent();
    }
    elseif (method_exists($form_object, 'getEntity')) {
      /** @var \Drupal\block\Entity\Block $entity */
      $entity = $form_object->getEntity();
    }

    return $entity;
  }

  /**
   * Get the Block config settings from the form state.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return array
   *   A list of settings set on the block configuration.
   */
  public function getBlockSettings(FormStateInterface $form_state) {
    $settings = [];
    $form_values = $form_state->getUserInput();

    $block_edit_forms = [
      'block_form',
      'layout_builder_add_block',
      'layout_builder_update_block',
    ];

    if (!empty($form_values['form_id']) && !in_array($form_values['form_id'], $block_edit_forms)) {
      $block = $this->getBlockConfigEntity($form_state);
      $settings = $block->get('settings') ?? $block->get('configuration');
    }
    elseif (!empty($form_values['settings'])) {
      $settings = $form_values['settings'];
    }

    return $settings;
  }

  /**
   * A single submitted style from the form state of the currently saved entity.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $plugin
   *   The machine name of the Block Style Plugin.
   * @param string $style_key
   *   The optional name of the specific style wanted.
   *
   * @return mixed|null
   *   A single style fetched from third party settings.
   */
  public function getBlockSubmittedStyle(FormStateInterface $form_state, $plugin, $style_key = NULL) {
    $styles = $this->getBlockSubmittedStyles($form_state, $plugin);
    return $styles[$style_key] ?? NULL;
  }

  /**
   * Get the submitted styles from the form state of the currently saved entity.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $plugin
   *   The machine name of the Block Style Plugin.
   *
   * @return mixed|null
   *   A list of styles fetched from third party settings.
   */
  public function getBlockSubmittedStyles(FormStateInterface $form_state, $plugin) {
    $form_values = $form_state->getUserInput();

    if ($form_values['form_id'] !== 'block_form') {
      $block = $this->getBlockConfigEntity($form_state);
      $styles = $block->getThirdPartySetting('block_style_plugins', $plugin);
    }
    else {
      $styles = $form_values['third_party_settings']['block_style_plugins'][$plugin] ?? NULL;
    }

    return $styles;
  }

  /**
   * Get the Section config entity from the form state.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return \Drupal\layout_builder\Section|null
   *   The entity for the current layout builder Section if it exists.
   *
   * @todo This can be further simplified once the following issue is committed
   * https://www.drupal.org/project/drupal/issues/3044117.
   */
  public function getLayoutSection(FormStateInterface $form_state) {
    /** @var \Drupal\layout_builder\Form\ConfigureSectionForm $form_object */
    $form_object = $form_state->getFormObject();

    // If using the https://www.drupal.org/project/drupal/issues/3044117 patch,
    // then this can use the built in method.
    if (method_exists($form_object, 'getCurrentSection')) {
      return $form_object->getCurrentSection();
    }

    // Sad hack to try to get the current section since a getCurrentSection()
    // does not exist.
    $build_info = $form_state->getBuildInfo();
    $section_storage = $form_object->getSectionStorage();
    $sections = $section_storage->getSections();
    $delta = (int) $build_info['args'][1];

    return (array_key_exists($delta, $sections)) ? $sections[$delta] : NULL;
  }

  /**
   * Determine if the block is an embedded inline block for layout builder.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   Return TRUE if the block is embedded in the layout builder.
   */
  public function isEmbeddedInlineBlock(FormStateInterface $form_state): bool {
    $component_config = [];

    $storage = $form_state->getStorage();
    if (isset($storage['layout_builder__component'])) {
      $component_config = $storage['layout_builder__component']->get('configuration');
    }
    else {
      $callback = $form_state->getBuildInfo()['callback_object'];
      if (method_exists($callback, 'getCurrentComponent')) {
        $block = $callback->getCurrentComponent();
        $component_config = $block->get('configuration');
      }
    }

    if (empty($component_config['id'])) {
      return FALSE;
    }

    return str_starts_with($component_config['id'], 'inline_block:');
  }

}
