<?php

namespace Drupal\block_style_plugins;

use Drupal\Core\Form\SubformState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;

/**
 * Base class for Section style plugins.
 */
class SectionFormAlter extends BaseFormAlter {

  protected $groupName = 'Section Styles';

  /**
   * Alter a form and set Layout Builder Section configuration.
   *
   * This code will be run as part of a form alter so that the current section
   * configuration will be available to the form.
   *
   * @param array $form
   *   The form definition array for the section configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function alterForm(array &$form, FormStateInterface $form_state) {
    // Check if this is a new section. Exit if this is a new section and the
    // patch from https://www.drupal.org/project/drupal/issues/3044117 has not
    // been applied.
    $new_section = strpos($form['#attributes']['data-layout-builder-target-highlight-id'], 'update') === FALSE;
    if (!method_exists('\Drupal\layout_builder\Form\ConfigureSectionForm', 'getCurrentSection') && $new_section) {
      return;
    }

    // Get the current section config entity. Exit if there is no section.
    $section = $this->getLayoutSection($form_state);
    if (!$section) {
      return;
    }

    // Find the section ID.
    $section_plugin_id = $section->getLayoutId();

    // Retrieve a list of style plugin definitions.
    $style_plugins = $this->blockStyleManager->getSectionDefinitions();

    foreach ($style_plugins as $plugin_id => $plugin_definition) {
      /** @var \Drupal\block_style_plugins\Plugin\BlockStyleInterface $style_plugin */
      $style_plugin = $this->blockStyleManager->createInstance($plugin_id);

      // Check to see if this should only apply to includes or if it has been
      // excluded.
      if ($this->allowStyles($section_plugin_id, $plugin_definition)) {
        // Create containers to place each plugin style settings into the styles
        // fieldset.
        $this->createGroup($form, $plugin_id, $plugin_definition);

        $styles = $section->getThirdPartySetting('block_style_plugins', $plugin_id, []);
        $style_plugin->setConfiguration($styles);

        // Allow plugins to add field elements to this form.
        if ($style_plugin instanceof PluginFormInterface) {
          $subform_state = SubformState::createForSubform($form['third_party_settings']['block_style_plugins'][$plugin_id], $form, $form_state);
          $form['third_party_settings']['block_style_plugins'][$plugin_id] += $style_plugin->buildConfigurationForm($form['third_party_settings']['block_style_plugins'][$plugin_id], $subform_state);
        }
      }
    }

    $this->addFormHandlers($form);
  }

  /**
   * Validate the section form.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function validateForm(array $form, FormStateInterface $form_state) {
    // Retrieve a list of possible style plugin definitions.
    $available_plugins = $this->blockStyleManager->getSectionDefinitions();
    $style_settings = $form_state->getValue(['third_party_settings', 'block_style_plugins'], []);

    foreach ($style_settings as $plugin_id => $settings) {
      // Only instantiate plugins that are available.
      if (array_key_exists($plugin_id, $available_plugins)) {
        /** @var \Drupal\block_style_plugins\Plugin\BlockStyleInterface $style_plugin */
        $style_plugin = $this->blockStyleManager->createInstance($plugin_id);
        if ($style_plugin instanceof PluginFormInterface) {
          $subform_state = SubformState::createForSubform($form['third_party_settings']['block_style_plugins'][$plugin_id], $form, $form_state);
          $style_plugin->validateConfigurationForm($form['third_party_settings']['block_style_plugins'][$plugin_id], $subform_state);
        }
      }
    }
  }

  /**
   * Submit the section form and save configuration.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array $form, FormStateInterface $form_state) {
    // Retrieve a list of possible style plugin definitions.
    $available_plugins = $this->blockStyleManager->getSectionDefinitions();
    $style_settings = $form_state->getValue(['third_party_settings', 'block_style_plugins'], []);
    $section = $this->getLayoutSection($form_state);

    foreach ($style_settings as $plugin_id => $styles) {
      // Only instantiate plugins that are available.
      if (array_key_exists($plugin_id, $available_plugins)) {
        /** @var \Drupal\block_style_plugins\Plugin\BlockStyleInterface $style_plugin */
        $style_plugin = $this->blockStyleManager->createInstance($plugin_id);
        $style_plugin->setConfiguration($styles);

        if ($style_plugin instanceof PluginFormInterface) {
          $subform_state = SubformState::createForSubform($form['third_party_settings']['block_style_plugins'][$plugin_id], $form, $form_state);
          $style_plugin->submitConfigurationForm($form['third_party_settings']['block_style_plugins'][$plugin_id], $subform_state);
          // Save any altered styles back to the plugin config.
          $altered_style = $form_state->getValue(['third_party_settings', 'block_style_plugins', $plugin_id]);
          $style_plugin->setConfiguration($altered_style);
        }

        // Remove any empty styles.
        $remaining_styles = $this->removeEmptyStyles($form, $style_plugin);
        if (empty($remaining_styles)) {
          $section->unsetThirdPartySetting('block_style_plugins', $plugin_id);
        }
        else {
          // Save the style to the Section.
          $section->setThirdPartySetting('block_style_plugins', $plugin_id, $style_plugin->getConfiguration());
        }
      }
    }
  }

}
