<?php

namespace Drupal\Tests\block_style_plugins\Unit;

use Drupal\block\BlockForm;
use Drupal\block\Entity\Block;
use Drupal\block_content\BlockContentInterface;
use Drupal\block_style_plugins\Form\ConfigureStyles;
use Drupal\block_style_plugins\FormEntityHelperTrait;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Entity\RevisionableStorageInterface;
use Drupal\Core\Form\FormState;
use Drupal\layout_builder\Form\AddBlockForm;
use Drupal\layout_builder\Form\ConfigureSectionForm;
use Drupal\layout_builder\Form\UpdateBlockForm;
use Drupal\layout_builder\Section;
use Drupal\layout_builder\SectionComponent;
use Drupal\Tests\UnitTestCase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * @coversDefaultClass \Drupal\block_style_plugins\FormEntityHelperTrait
 * @group block_style_plugins
 */
class FormEntityHelperTraitTest extends UnitTestCase {

  /**
   * Mocked entity repository service.
   *
   * @var \Drupal\Core\Entity\EntityRepository
   */
  protected $entityRepository;

  /**
   * Mocked entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Mocked form state.
   *
   * @var \Drupal\Core\Form\FormStateInterface
   */
  protected $formState;

  /**
   * Mocked Block Plugin.
   *
   * @var \Drupal\Core\Block\BlockPluginInterface
   */
  protected $blockPlugin;

  /**
   * Mocked Block Plugin.
   *
   * @var \Drupal\block\Entity\Block
   */
  protected $block;

  /**
   * Class with the applied trait.
   *
   * @var \Drupal\block_style_plugins\FormEntityHelperTrait
   */
  protected $trait;

  /**
   * Create the setup for constants and configFactory stub.
   */
  protected function setUp(): void {
    parent::setUp();

    // Stub the Entity Repository Service.
    $this->entityRepository = $this->prophesize(EntityRepositoryInterface::class);

    // Stub the Entity Type Manager.
    $this->entityTypeManager = $this->prophesize(EntityTypeManagerInterface::class);

    // Block plugin.
    $this->blockPlugin = $this->prophesize(BlockPluginInterface::class);
    $this->blockPlugin->getBaseId()->willReturn('block_content');
    $this->blockPlugin->getDerivativeId()->willReturn('uuid-1234');
    $this->blockPlugin->getPluginId()->willReturn('basic_block');

    $this->block = $this->prophesize(Block::class);
    $this->block->getPlugin()->willReturn($this->blockPlugin->reveal());

    $blockForm = $this->prophesize(BlockForm::class);
    $blockForm->getEntity()->willReturn($this->block->reveal());

    $this->formState = (new FormState())
      ->setFormObject($blockForm->reveal());

    $this->trait = new class($this->entityRepository->reveal(), $this->entityTypeManager->reveal()) {
      use FormEntityHelperTrait;

      public function __construct($entityRepository, $entityTypeManager) {
        $this->entityRepository = $entityRepository;
        $this->entityTypeManager = $entityTypeManager;
      }
    };

  }

  /**
   * Tests the getBlockContentBundle method.
   */
  public function testGetBlockContentBundle() {
    // Block content.
    $entity = $this->prophesize(BlockContentInterface::class);
    $entity->bundle()->willReturn('basic_custom_block');

    $this->entityRepository->loadEntityByUuid('block_content', 'uuid-1234')
      ->willReturn($entity->reveal());

    $bundle = $this->trait->getBlockContentBundle($this->formState);
    $this->assertEquals('basic_custom_block', $bundle);
  }

  /**
   * Tests the getBlockContent method.
   */
  public function testGetBlockContent() {
    // Block Content.
    $entity = $this->prophesize(BlockContentInterface::class);
    $this->entityRepository->loadEntityByUuid('block_content', 'uuid-1234')
      ->willReturn($entity->reveal());

    $return = $this->trait->getBlockContent($this->formState);
    $this->assertInstanceOf('\Drupal\block_content\BlockContentInterface', $return);

    // Inline Block content.
    $this->blockPlugin->getBaseId()->willReturn('inline_block');
    $this->blockPlugin->getConfiguration()
      ->willReturn(['block_serialized' => serialize('my_entity')]);

    $return = $this->trait->getBlockContent($this->formState);
    $this->assertEquals('my_entity', $return);

    // Inline Block Content revision.
    $this->blockPlugin->getBaseId()->willReturn('inline_block');
    $this->blockPlugin->getConfiguration()->willReturn(['block_revision_id' => '123']);

    $storage = $this->prophesize(RevisionableStorageInterface::class);
    $storage->loadRevision('123')->willReturn($entity->reveal());
    $this->entityTypeManager->getStorage('block_content')->willReturn($storage);

    $return = $this->trait->getBlockContent($this->formState);
    $this->assertInstanceOf('\Drupal\block_content\BlockContentInterface', $return);
  }

  /**
   * Tests the getEntity method.
   */
  public function testGetEntity() {
    // Block.
    $result = $this->trait->getEntity($this->formState);
    $this->assertInstanceOf('\Drupal\block\Entity\Block', $result);

    // Section.
    $section = $this->prophesize(Section::class);
    $section_form = $this->prophesize(ConfigureSectionForm::class);
    $section_form->getCurrentSection()->willReturn($section->reveal());
    $this->formState->setFormObject($section_form->reveal());

    $result = $this->trait->getEntity($this->formState);
    $this->assertInstanceOf('\Drupal\layout_builder\Section', $result);
  }

  /**
   * Tests the getBlockConfigEntity method.
   */
  public function testGetBlockConfigEntity() {
    // 1. Normal block form.
    $result = $this->trait->getBlockConfigEntity($this->formState);
    $this->assertInstanceOf('\Drupal\block\Entity\Block', $result);

    // 2. Layout Builder embedded "add" form.
    $entity = $this->prophesize(SectionComponent::class);
    $blockForm = $this->prophesize(AddBlockForm::class);
    $this->formState->set('layout_builder__component', $entity->reveal());

    $this->formState->setFormObject($blockForm->reveal());
    $result = $this->trait->getBlockConfigEntity($this->formState);
    $this->assertInstanceOf('\Drupal\layout_builder\SectionComponent', $result);

    // 3. Layout Builder embedded "update" form.
    $entity = $this->prophesize(SectionComponent::class);
    $blockForm = $this->prophesize(UpdateBlockForm::class);
    $blockForm->getCurrentComponent()->willReturn($entity->reveal());

    $this->formState->setFormObject($blockForm->reveal());
    $result = $this->trait->getBlockConfigEntity($this->formState);
    $this->assertInstanceOf('\Drupal\layout_builder\SectionComponent', $result);

    // 4. Layout Builder tray form.
    $entity = $this->prophesize(SectionComponent::class);
    $blockForm = $this->prophesize(ConfigureStyles::class);
    $blockForm->getComponent()->willReturn($entity->reveal());

    $this->formState->setFormObject($blockForm->reveal());
    $result = $this->trait->getBlockConfigEntity($this->formState);
    $this->assertInstanceOf('\Drupal\layout_builder\SectionComponent', $result);
  }

  /**
   * Tests the getBlockSettings method.
   */
  public function testGetBlockSettings() {
    // Normal Block form.
    $this->formState->setUserInput([
      'form_id' => 'block_form',
      'settings' => ['setting1'],
    ]);

    $result = $this->trait->getBlockSettings($this->formState);
    $this->assertEquals(['setting1'], $result);

    // Layout Builder.
    $this->formState->setUserInput([
      'form_id' => 'layout_builder',
      'settings' => ['setting2'],
    ]);
    $this->block->get('settings')->willReturn(['setting2']);

    $result = $this->trait->getBlockSettings($this->formState);
    $this->assertEquals(['setting2'], $result);

    // Layout Builder style tray.
    $entity = $this->prophesize(SectionComponent::class);
    $entity->get('configuration')->willReturn(['setting3']);
    $entity->get('settings')->willReturn(NULL);

    $blockForm = $this->prophesize(ConfigureStyles::class);
    $blockForm->getComponent()->willReturn($entity->reveal());

    $this->formState->setFormObject($blockForm->reveal());
    $this->formState->setUserInput([
      'form_id' => 'layout_builder',
      'settings' => [],
    ]);

    $result = $this->trait->getBlockSettings($this->formState);
    $this->assertEquals(['setting3'], $result);
  }

  /**
   * Tests the getBlockSubmittedStyle method.
   */
  public function testGetBlockSubmittedStyle() {
    $this->formState->setUserInput([
      'form_id' => 'block_form',
      'third_party_settings' => [
        'block_style_plugins' => [
          'plugin_name' => [
            'first_option' => 'style1',
            'second_option' => 'style2',
          ]
        ]
      ]
    ]);

    $result = $this->trait->getBlockSubmittedStyle($this->formState, 'plugin_name', 'missing_style');
    $this->assertNull($result);

    $result = $this->trait->getBlockSubmittedStyle($this->formState, 'plugin_name', 'first_option');
    $this->assertEquals('style1', $result);

    $result = $this->trait->getBlockSubmittedStyle($this->formState, 'plugin_name', 'second_option');
    $this->assertEquals('style2', $result);
  }

  /**
   * Tests the getBlockSubmittedStyles method.
   */
  public function testGetBlockSubmittedStyles() {
    // Normal Block form.
    $this->formState->setUserInput([
      'form_id' => 'block_form',
      'third_party_settings' => [
        'block_style_plugins' => [
          'plugin_name' => 'my_style'
        ]
      ]
    ]);

    $result = $this->trait->getBlockSubmittedStyles($this->formState, 'missing_plugin_name');
    $this->assertNull($result);

    $result = $this->trait->getBlockSubmittedStyles($this->formState, 'plugin_name');
    $this->assertEquals('my_style', $result);

    // Layout Builder.
    $this->formState->setUserInput([
      'form_id' => 'layout_builder',
      'third_party_settings' => [
        'block_style_plugins' => [
          'plugin_name1' => 'my_style_in_lb'
        ]
      ]
    ]);

    $this->block->getThirdPartySetting('block_style_plugins', 'plugin_name')->willReturn('my_style_in_lb');

    $result = $this->trait->getBlockSubmittedStyles($this->formState, 'plugin_name');
    $this->assertEquals('my_style_in_lb', $result);
  }

  /**
   * Tests the getLayoutSection method.
   */
  public function testGetLayoutSection() {
    $section = $this->prophesize(Section::class);

    $section_form = $this->prophesize(ConfigureSectionForm::class);
    $section_form->getCurrentSection()->willReturn($section->reveal());
    $section_form->getCurrentSection()->shouldBeCalled();

    $this->formState->setFormObject($section_form->reveal());

    $result = $this->trait->getLayoutSection($this->formState);

    $this->assertInstanceOf('\Drupal\layout_builder\Section', $result);
  }

}
