<?php

namespace Drupal\Tests\block_style_plugins\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\block_style_plugins\Plugin\BlockStyleManager;
use Drupal\block_style_plugins\SectionFormAlter;
use Drupal\layout_builder\Form\ConfigureSectionForm;
use Drupal\layout_builder\Section;
use Drupal\layout_builder\SectionStorageInterface;
use Drupal\Tests\block_style_plugins\Unit\Plugin\MockBlockStyleBase;

/**
 * @coversDefaultClass \Drupal\block_style_plugins\SectionFormAlter
 * @group block_style_plugins
 */
class SectionFormAlterTest extends UnitTestCase {

  /**
   * Mocked entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Instance of the Block Style Manager service.
   *
   * @var \Drupal\block_style_plugins\Plugin\BlockStyleManager
   */
  protected $blockStyleManager;

  /**
   * Mocked form state.
   *
   * @var \Drupal\Core\Form\FormStateInterface
   */
  protected $formState;

  /**
   * Mocked Section.
   *
   * @var \Drupal\layout_builder\Section
   */
  protected $section;

  /**
   * Instance of the MockBlockStyleBase plugin.
   *
   * @var \Drupal\Tests\block_style_plugins\Unit\Plugin\MockBlockStyleBase
   */
  protected $plugin;

  /**
   * Instance of the BlockFormAlter.
   *
   * @var \Drupal\block_style_plugins\BlockFormAlter
   */
  protected $classInstance;

  /**
   * Create the setup for constants and configFactory stub.
   */
  protected function setUp(): void {
    parent::setUp();

    // Stub the Entity Type Manager.
    $this->entityTypeManager = $this->prophesize(EntityTypeManagerInterface::class);

    // Form state double.
    $this->formState = $this->prophesize(FormStateInterface::class);

    // Block plugin.
    $this->section = $this->prophesize(Section::class);
    $this->section->getLayoutId()->willReturn('layout_onecol');

    $configuration = [];
    $plugin_id = 'block_style_plugins';
    $plugin_definition['provider'] = 'block_style_plugins';
    $plugin_definition['type'] = 'section';

    $this->plugin = new MockBlockStyleBase(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $this->entityTypeManager->reveal()
    );

    // Stub the Block Style Manager service.
    $this->blockStyleManager = $this->prophesize(BlockStyleManager::class);
    $this->blockStyleManager->getSectionDefinitions()
      ->willReturn([$plugin_id => $plugin_definition]);
    $this->blockStyleManager->createInstance('block_style_plugins')
      ->willReturn($this->plugin);

    $this->classInstance = new SectionFormAlter(
      $this->blockStyleManager->reveal()
    );

    // Create a translation stub for the t() method.
    $translator = $this->getStringTranslationStub();
    $this->classInstance->setStringTranslation($translator);
  }

  /**
   * Tests the create method.
   *
   * @see ::create()
   */
  public function testCreate() {
    $container = $this->prophesize(ContainerInterface::class);
    $container->get('plugin.manager.block_style.processor')
      ->willReturn($this->blockStyleManager->reveal());

    $instance = SectionFormAlter::create(
      $container->reveal()
    );
    $this->assertInstanceOf('Drupal\block_style_plugins\SectionFormAlter', $instance);
  }

  /**
   * Tests the formAlter() method.
   *
   * @see ::formAlter()
   */
  public function testFormAlter() {
    $this->section->getThirdPartySetting('block_style_plugins', 'block_style_plugins', [])
      ->willReturn(['test_style' => TRUE]);

    $sectionStorage = $this->prophesize(SectionStorageInterface::class);
    $sectionStorage->getSections()->willReturn([$this->section->reveal()]);

    $sectionForm = $this->prophesize(ConfigureSectionForm::class);
    $sectionForm->getSectionStorage()->willReturn($sectionStorage->reveal());
    // If has patch from: https://www.drupal.org/project/drupal/issues/3044117.
    $sectionForm->getCurrentSection()->willReturn($this->section->reveal());

    $this->formState->getFormObject()->willReturn($sectionForm->reveal());
    $this->formState->getBuildInfo()->willReturn(['args' => [1 => 0]]);

    $form = [
      '#attributes' => [
        'data-layout-builder-target-highlight-id' => 'section-update-1',
      ],
      'actions' => [
        'submit' => [
          '#validate' => [],
          '#submit' => [],
        ],
      ]
    ];
    $this->classInstance->alterForm($form, $this->formState->reveal());

    // Check that a block_styles array is set.
    $this->assertArrayHasKey('block_styles', $form);

    // Check that styles were set.
    $styles = $this->plugin->getConfiguration();
    $expected_styles = [
      'sample_class' => '',
      'sample_checkbox' => '',
      'test_style' => TRUE,
    ];
    $this->assertEquals($expected_styles, $styles);

    // Check third party settings.
    $expected_third_party_settings['block_style_plugins']['block_style_plugins'] = [
      '#type' => 'container',
      '#group' => 'block_styles',
    ];
    $this->assertEquals($expected_third_party_settings, $form['third_party_settings']);

    // Check that validation and submit callbacks are set.
    $action = $form['actions']['submit'];
    $this->assertInstanceOf('Drupal\block_style_plugins\SectionFormAlter', $action['#validate'][0][0]);
    $this->assertEquals('validateForm', $action['#validate'][0][1]);
    $this->assertInstanceOf('Drupal\block_style_plugins\SectionFormAlter', $action['#submit'][0][0]);
    $this->assertEquals('submitForm', $action['#submit'][0][1]);
  }

}
